testCase = { // Test case 1: No remote control.
  [] call testCases_createUnits;

  {
    [] call _x;
  } forEach (_this select 0);

  a2 allowDamage true;

  private _timeout = time + 5;

  waitUntil { !alive a2 || { time > _timeout } };

  {
    [] call _x;
  } forEach (_this select 1);

  [!alive a2, {
    deleteVehicle a1;
    deleteVehicle a2;
  }];
};

testCaseWithRC = [
  {
    sleep 0.2;
  },{
    private _handle = (time + 2) spawn {
      while { time < _this } do {
        a1 setDir 179;
      };
    };

    player remoteControl a1;
    waitUntil { scriptDone _handle };

    objNull remoteControl a1;
    player remoteControl objNull;
  }
];

testCase1 = { // Without RC, with camera
  [[{
    a1 switchCamera "GUNNER";
  }],[]] call testCase;
};

testCase2 = { // Without RC, without camera
  [[],[]] call testCase;
};

testCase3 = { // With RC, with camera
  [[
    testCaseWithRC select 0,
    { a1 switchCamera "GUNNER"; },
    testCaseWithRC select 1,
    { a1 switchCamera "GUNNER"; }
  ],[]] call testCase;
};

testCase4 = { // With RC, without camera
  [[
    testCaseWithRC select 0,
    testCaseWithRC select 1
  ],[]] call testCase;
};

testCaseEnableSimulation = {
  [[
    testCaseWithRC select 0,
    { a1 switchCamera "GUNNER"; },
    testCaseWithRC select 1,
    {
      a1 switchCamera "GUNNER";
      a1 enableSimulation false;
      sleep 0.1;
      a1 enableSimulation true;
    }
  ],[]] call testCase;
};

testCaseDynamicSimulation = {
  [[
    testCaseWithRC select 0,
    { a1 switchCamera "GUNNER"; },
    testCaseWithRC select 1,
    {
      a1 switchCamera "GUNNER";
      enableDynamicSimulationSystem false;
      sleep 0.1;
      a1 enableSimulation true;
    }
  ],[{
    enableDynamicSimulationSystem true;
  }]] call testCase;
};

testCases = [
  ["No RC", testCase1],
  ["With RC", testCase3],
  ["With RC (no camera)", testCase4],
  ["With RC (enableSimulation)", testCaseEnableSimulation],
  ["With RC (no dynamic simulation)", testCaseDynamicSimulation]
];

testCaseResults = [];
{
  testCaseResults pushBack "QUEUED";
} forEach testCases;

[] spawn {
  while {true} do {
    if (!isNil "a1" && { !isNull a1 }) then {
      a1 setVehicleAmmo 1.0;
    };
    sleep 0.1;
  };
};

[] spawn {
  while {true} do {
    private _text = "<t size='2.0'>Test cases</t>";
    {
      private _desc = _x select 0;
      private _result = testCaseResults select _forEachIndex;
      private _color = "#FFFF00";
      if (_result == "RUNNING") then {
        _color = "#AAAAFF";
      } else {
        if (_result == "FAILED") then {
          _color = "#FF0000";
        } else {
          if (_result == "PASSED") then {
            _color = "#00FF00";
          };
        };
      };
      _text = _text + format ["<br/>%4 (%1): <t color='%2'>%3</t>", _forEachIndex, _color, _result, _desc];
    } forEach testCases;
    hintSilent parseText _text;
    sleep 0.1;
  };
};

testCases_createUnits = {
  if (!isNil "a1" && { !isNull a1 }) then {
    deleteVehicle a1;
  };
  waitUntil { isNil "a1" || { isNull a1 } };
  "B_Soldier_F" createUnit [getMarkerPos "start_marker", createGroup [west, true], "a1 = this;"];

  if (!isNil "a2" && { !isNull a2 }) then {
    deleteVehicle a2;
  };
  waitUntil { isNil "a2" || { isNull a2 } };
  "O_Soldier_F" createUnit [getMarkerPos "target_marker", createGroup [east, true], "a2 = this; this allowDamage false; this disableAI ""ALL"";"];
  waitUntil { !isNull a2 && { !isNull a1 } };

  a1 setDir 179;
  a2 setDir 180;

};

[] spawn {
  sleep 1;
  {
    testCaseResults set [_forEachIndex, "RUNNING"];
    private _results = [] call (_x select 1);
    private _result = _results select 0;
    if (_result) then {
      _result = "PASSED";
    } else {
      _result = "FAILED";
    };
    testCaseResults set [_forEachIndex, _result];
    
    sleep 1;
    call (_results select 1);
    sleep 0.1;
  } forEach testCases;
};
